@@ -3,7 +3,7 @@
 #
 # BSD LICENSE
 # 
-# Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
+# Copyright (c) 2014-2017, Michael Truog <mjtruog at gmail dot com>
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '1.54';
+$CloudI::API::VERSION = '1.60';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -587,8 +587,13 @@ sub _callback
             {
                 $response = $response_info;
                 $response_info = '';
+                if (! defined($response))
+                {
+                    $response = '';
+                }
             }
-            elsif (ref($response_info) ne '')
+            elsif (ref($response_info) ne '' ||
+                   ! defined($response_info))
             {
                 $response_info = '';
             }
@@ -654,8 +659,13 @@ sub _callback
             {
                 $response = $response_info;
                 $response_info = '';
+                if (! defined($response))
+                {
+                    $response = '';
+                }
             }
-            elsif (ref($response_info) ne '')
+            elsif (ref($response_info) ne '' ||
+                   ! defined($response_info))
             {
                 $response_info = '';
             }
