@@ -44,7 +44,7 @@
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2009-2016 Michael Truog
-%%% @version 1.5.4 {@date} {@time}
+%%% @version 1.5.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_logger).
@@ -100,7 +100,7 @@
         destination = undefined
             :: undefined | ?MODULE | {?MODULE, node()},
         syslog = undefined
-            :: undefined | port(),
+            :: undefined | pid(),
         syslog_level = undefined
             :: undefined | cloudi_service_api:loglevel(),
         formatters
@@ -155,7 +155,7 @@ set(#config_logging{file = FilePath} = LoggingConfig) ->
         FilePath =:= undefined ->
             gen_server:cast(?MODULE, {set, LoggingConfig});
         is_list(FilePath) ->
-            case file:open(FilePath, [append, raw]) of
+            case file:open(FilePath, [raw, append]) of
                 {ok, Fd} ->
                     file:close(Fd),
                     gen_server:cast(?MODULE, {set, LoggingConfig});
@@ -177,7 +177,7 @@ file_set(undefined) ->
     gen_server:cast(?MODULE, {file_set, undefined});
 file_set(FilePath)
     when is_list(FilePath) andalso is_integer(hd(FilePath)) ->
-    case file:open(FilePath, [append, raw]) of
+    case file:open(FilePath, [raw, append]) of
         {ok, Fd} ->
             file:close(Fd),
             gen_server:cast(?MODULE, {file_set, FilePath});
@@ -506,11 +506,8 @@ init([#config_logging{file = FilePath,
     #state{syslog_level = SyslogLevel} = StateNext = case SyslogConfig of
         undefined ->
             State;
-        #config_logging_syslog{identity = SyslogIdentity,
-                               facility = SyslogFacility,
-                               level = SyslogLevel0} ->
-            ok = cloudi_x_syslog:load(),
-            State#state{syslog = syslog_open(SyslogIdentity, SyslogFacility),
+        #config_logging_syslog{level = SyslogLevel0} ->
+            State#state{syslog = syslog_open(SyslogConfig),
                         syslog_level = SyslogLevel0}
     end,
     Level = log_level([FileLevel, SyslogLevel, FormattersLevel]),
@@ -664,14 +661,8 @@ handle_info(Request, State) ->
 
 terminate(_, #state{fd = Fd,
                     syslog = Syslog}) ->
-    file:close(Fd),
-    if
-        is_port(Syslog) ->
-            (catch cloudi_x_syslog:close(Syslog)),
-            cloudi_x_syslog:unload();
-        true ->
-            ok
-    end,
+    _ = (catch file:close(Fd)),
+    ok = syslog_close(Syslog),
     ok.
 
 code_change(_, State, _) ->
@@ -776,31 +767,13 @@ log_config_syslog_set(SyslogConfig,
             SyslogLevelNew0
     end,
     SwitchF = fun(StateSwitch) ->
-        if
-            SyslogOld =:= undefined ->
-                if
-                    SyslogConfig =:= undefined ->
-                        ok;
-                    true ->
-                        ok = cloudi_x_syslog:load()
-                end;
-            is_port(SyslogOld) ->
-                ok = cloudi_x_syslog:close(SyslogOld),
-                if
-                    SyslogConfig =:= undefined ->
-                        ok = cloudi_x_syslog:unload();
-                    true ->
-                        ok
-                end
-        end,
+        ok = syslog_close(SyslogOld),
         case SyslogConfig of
             undefined ->
                 StateSwitch#state{syslog = undefined,
                                   syslog_level = undefined};
-            #config_logging_syslog{identity = SyslogIdentity,
-                                   facility = SyslogFacility} ->
-                StateSwitch#state{syslog = syslog_open(SyslogIdentity,
-                                                       SyslogFacility),
+            #config_logging_syslog{} ->
+                StateSwitch#state{syslog = syslog_open(SyslogConfig),
                                   syslog_level = SyslogLevelNew}
         end
     end,
@@ -1168,7 +1141,7 @@ log_message_internal(Level, Timestamp, Node, Pid,
     end,
     {SyslogResult, StateN} = case log_level_allowed(SyslogLevel, Level) of
         true ->
-            log_syslog(Level, Message, State1);
+            log_syslog(Level, Timestamp, Message, State1);
         false ->
             {ok, State1}
     end,
@@ -1190,7 +1163,7 @@ log_file(Message,
          #state{file_path = FilePath,
                 fd = FdOld,
                 inode = InodeOld} = State) ->
-    case file:read_file_info(FilePath) of
+    case file:read_file_info(FilePath, [raw]) of
         {ok, #file_info{inode = CurrentInode}} ->
             if
                 CurrentInode == InodeOld ->
@@ -1228,9 +1201,9 @@ log_file(Message,
 log_file_open(undefined, State) ->
     {ok, State};
 log_file_open(FilePath, State) ->
-    case file:open(FilePath, [append, raw]) of
+    case file:open(FilePath, [raw, append]) of
         {ok, Fd} ->
-            case file:read_file_info(FilePath) of
+            case file:read_file_info(FilePath, [raw]) of
                 {ok, #file_info{inode = Inode}} ->
                     {ok, State#state{file_path = FilePath,
                                      fd = Fd,
@@ -1243,7 +1216,7 @@ log_file_open(FilePath, State) ->
     end.
 
 log_file_reopen(FilePath, Inode, State) ->
-    case file:open(FilePath, [append, raw]) of
+    case file:open(FilePath, [raw, append]) of
         {ok, Fd} ->
             {ok, State#state{file_path = FilePath,
                              fd = Fd,
@@ -1252,10 +1225,11 @@ log_file_reopen(FilePath, Inode, State) ->
             Error
     end.
 
-log_syslog(Level, Message,
+log_syslog(Level, Timestamp, Message,
            #state{syslog = Syslog} = State) ->
     SyslogPriority = log_level_to_syslog_priority(Level),
-    cloudi_x_syslog:log(Syslog, SyslogPriority, Message),
+    ok = cloudi_x_syslog_socket:send(Syslog, SyslogPriority,
+                                     Timestamp, Message),
     {ok, State}.
 
 log_redirect(_, Destination,
@@ -1328,17 +1302,17 @@ log_level_to_string(trace) ->
     "TRACE".
 
 log_level_to_syslog_priority(fatal) ->
-    emerg;
+    critical;
 log_level_to_syslog_priority(error) ->
-    err;
+    error;
 log_level_to_syslog_priority(warn) ->
     warning;
 log_level_to_syslog_priority(info) ->
-    info;
+    notice;
 log_level_to_syslog_priority(debug) ->
-    debug;
+    informational;
 log_level_to_syslog_priority(trace) ->
-    8.
+    debug.
 
 log_level_allowed(trace, fatal) ->
     true;
@@ -1761,11 +1735,32 @@ load_interface_module(Level, Mode, Process) when is_atom(Level) ->
             Error
     end.
 
-syslog_open(SyslogIdentity, SyslogFacility) ->
-    {ok, Syslog} = cloudi_x_syslog:open(SyslogIdentity,
-                                        [ndelay, pid], SyslogFacility),
+syslog_open(#config_logging_syslog{identity = SyslogIdentity,
+                                   facility = SyslogFacility,
+                                   transport = SyslogTransport,
+                                   transport_options = SyslogTransportOptions,
+                                   protocol = SyslogProtocol,
+                                   path = SyslogPath,
+                                   host = SyslogHost,
+                                   port = SyslogPort}) ->
+    Options = [{app_name, SyslogIdentity},
+               {facility, SyslogFacility},
+               {transport, SyslogTransport},
+               {transport_options, SyslogTransportOptions},
+               {protocol, SyslogProtocol},
+               {utf8, true},
+               {path, SyslogPath},
+               {host, SyslogHost},
+               {port, SyslogPort},
+               {timeout, 5000}],
+    {ok, Syslog} = cloudi_x_syslog_socket:start_link(Options),
     Syslog.
 
+syslog_close(undefined) ->
+    ok;
+syslog_close(Syslog) when is_pid(Syslog) ->
+    cloudi_x_syslog_socket:stop_link(Syslog). % asynchronous stop
+
 aspects_log([], _, _, _, _, _, _, _, _, _, _) ->
     ok;
 aspects_log([{M, F} | L], Level, Timestamp, Node, Pid,
