@@ -9,7 +9,7 @@
 %%%
 %%% BSD LICENSE
 %%% 
-%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2017, Michael Truog <mjtruog at gmail dot com>
 %%% All rights reserved.
 %%% 
 %%% Redistribution and use in source and binary forms, with or without
@@ -44,8 +44,8 @@
 %%% DAMAGE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.4 {@date} {@time}
+%%% @copyright 2011-2017 Michael Truog
+%%% @version 1.5.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api).
@@ -78,14 +78,7 @@
          logging/1,
          code_path_add/2,
          code_path_remove/2,
-         code_path/1,
-         % deprecated, renamed to logging_level_set/2
-         loglevel_set/2,
-         % deprecated, renamed to logging_redirect_set/2
-         log_redirect/2]).
-
--deprecated([loglevel_set/2,
-             log_redirect/2]).
+         code_path/1]).
 
 -include("cloudi_service_api.hrl").
 -include("cloudi_core_i_constants.hrl").
@@ -109,21 +102,79 @@
     (?TIMEOUT_DELTA + 1)..?TIMEOUT_MAX_ERLANG.
 -export_type([dest_refresh_delay_milliseconds/0]).
 
--type timeout_milliseconds() ::
-    (?TIMEOUT_DELTA + 1)..?TIMEOUT_MAX.
--export_type([timeout_milliseconds/0]).
+-type timeout_initialize_value_milliseconds() ::
+    ?TIMEOUT_INITIALIZE_MIN..?TIMEOUT_INITIALIZE_MAX.
+-type timeout_initialize_milliseconds() ::
+    limit_min | limit_max | timeout_initialize_value_milliseconds().
+-export_type([timeout_initialize_value_milliseconds/0,
+              timeout_initialize_milliseconds/0]).
 
--type request_timeout_immediate_max_milliseconds() ::
-    0..?TIMEOUT_MAX_ERLANG.
--export_type([request_timeout_immediate_max_milliseconds/0]).
+-type timeout_send_async_value_milliseconds() ::
+    ?TIMEOUT_SEND_ASYNC_MIN..?TIMEOUT_SEND_ASYNC_MAX.
+-type timeout_send_async_milliseconds() ::
+    limit_min | limit_max | timeout_send_async_value_milliseconds().
+-export_type([timeout_send_async_value_milliseconds/0,
+              timeout_send_async_milliseconds/0]).
 
--type response_timeout_immediate_max_milliseconds() ::
-    0..?TIMEOUT_MAX_ERLANG.
--export_type([response_timeout_immediate_max_milliseconds/0]).
+-type timeout_send_sync_value_milliseconds() ::
+    ?TIMEOUT_SEND_SYNC_MIN..?TIMEOUT_SEND_SYNC_MAX.
+-type timeout_send_sync_milliseconds() ::
+    limit_min | limit_max | timeout_send_sync_value_milliseconds().
+-export_type([timeout_send_sync_value_milliseconds/0,
+              timeout_send_sync_milliseconds/0]).
 
--type timeout_terminate_milliseconds() ::
+-type timeout_terminate_value_milliseconds() ::
     ?TIMEOUT_TERMINATE_MIN..?TIMEOUT_TERMINATE_MAX.
--export_type([timeout_terminate_milliseconds/0]).
+-type timeout_terminate_milliseconds() ::
+    limit_min | limit_max | timeout_terminate_value_milliseconds().
+-export_type([timeout_terminate_value_milliseconds/0,
+              timeout_terminate_milliseconds/0]).
+
+-type request_timeout_immediate_max_value_milliseconds() ::
+    0..?TIMEOUT_MAX_ERLANG.
+-type request_timeout_immediate_max_milliseconds() ::
+    limit_min | limit_max | request_timeout_immediate_max_value_milliseconds().
+-export_type([request_timeout_immediate_max_value_milliseconds/0,
+              request_timeout_immediate_max_milliseconds/0]).
+
+-type response_timeout_immediate_max_value_milliseconds() ::
+    0..?TIMEOUT_MAX_ERLANG.
+-type response_timeout_immediate_max_milliseconds() ::
+    limit_min | limit_max | response_timeout_immediate_max_value_milliseconds().
+-export_type([response_timeout_immediate_max_value_milliseconds/0,
+              response_timeout_immediate_max_milliseconds/0]).
+
+-type restart_delay_value_milliseconds() ::
+    0..?TIMEOUT_MAX_ERLANG.
+-type restart_delay_milliseconds() ::
+    limit_min | limit_max | restart_delay_value_milliseconds().
+-export_type([restart_delay_value_milliseconds/0,
+              restart_delay_milliseconds/0]).
+
+-type latency_mean_time_value_milliseconds() ::
+    0..?TIMEOUT_MAX_ERLANG.
+-type latency_mean_time_milliseconds() ::
+    limit_min | limit_max | latency_mean_time_value_milliseconds().
+-type latency_min_time_value_milliseconds() ::
+    0..?TIMEOUT_MAX_ERLANG.
+-type latency_min_time_milliseconds() ::
+    limit_min | limit_max | latency_min_time_value_milliseconds().
+-type latency_max_time_value_milliseconds() ::
+    1..?TIMEOUT_MAX_ERLANG.
+-type latency_max_time_milliseconds() ::
+    limit_min | limit_max | latency_max_time_value_milliseconds().
+-type latency_time_value_milliseconds() ::
+    1..?TIMEOUT_MAX_ERLANG.
+-type latency_time_milliseconds() ::
+    limit_min | limit_max | latency_time_value_milliseconds().
+-export_type([latency_mean_time_value_milliseconds/0,
+              latency_mean_time_milliseconds/0,
+              latency_min_time_value_milliseconds/0,
+              latency_min_time_milliseconds/0,
+              latency_max_time_value_milliseconds/0,
+              latency_max_time_milliseconds/0,
+              latency_time_value_milliseconds/0,
+              latency_time_milliseconds/0]).
 
 -type acl() ::
     list(atom() | cloudi:service_name_pattern()).
@@ -141,14 +192,10 @@
     1..(?TIMEOUT_MAX_ERLANG div 1000).
 -export_type([period_seconds/0]).
 
--type latency_time_milliseconds() ::
-    1..?TIMEOUT_MAX_ERLANG.
--export_type([latency_time_milliseconds/0]).
-
 -type aspect_init_after_internal_f() ::
     fun((Args :: list(),
          Prefix :: cloudi_service:service_name_pattern(),
-         Timeout :: timeout_milliseconds(),
+         Timeout :: timeout_initialize_value_milliseconds(),
          State :: any(),
          Dispatcher :: cloudi_service:dispatcher()) ->
         {ok, NewState :: any()} |
@@ -156,7 +203,7 @@
 -type aspect_init_after_external_f() ::
     fun((CommandLine :: list(string()),
          Prefix :: cloudi:service_name_pattern(),
-         Timeout :: timeout_milliseconds(),
+         Timeout :: timeout_initialize_value_milliseconds(),
          State :: any()) ->
         {ok, NewState :: any()} |
         {stop, Reason :: any(), NewState :: any()}).
@@ -276,7 +323,7 @@
               aspect_info_after_internal/0]).
 -type aspect_terminate_f() ::
     fun((Reason :: any(),
-         Timeout :: timeout_milliseconds(),
+         Timeout :: timeout_terminate_value_milliseconds(),
          State :: any()) ->
         {ok, State :: any()}).
 -type aspect_terminate_before_internal_f() ::
@@ -296,6 +343,13 @@
               aspect_terminate_before_internal/0,
               aspect_terminate_before_external/0]).
 
+-type max_heap_size_options() ::
+    non_neg_integer() |
+    #{size => non_neg_integer(),
+      kill => boolean(),
+      error_logger => boolean()}.
+-export_type([max_heap_size_options/0]).
+
 -type limit_external_key() ::
     as | core | cpu | data | fsize | memlock | msgqueue | nice | nofile |
     nproc | rss | rtprio | rttime | sigpending | stack | vmem.
@@ -351,12 +405,19 @@
                {count_min, number()}) | false} |
          {timeout_terminate,
           undefined | timeout_terminate_milliseconds()} |
+         {restart_delay,
+          list({time_exponential_min, restart_delay_milliseconds()} |
+               {time_exponential_max, restart_delay_milliseconds()} |
+               {time_linear_min, restart_delay_milliseconds()} |
+               {time_linear_slope, restart_delay_milliseconds()} |
+               {time_linear_max, restart_delay_milliseconds()} |
+               {time_absolute, restart_delay_milliseconds()}) | false} |
          {scope, atom()} |
          {monkey_latency,
-          list({time_uniform_min, latency_time_milliseconds()} |
-               {time_uniform_max, latency_time_milliseconds()} |
-               {time_gaussian_mean, latency_time_milliseconds()} |
-               {time_gaussian_stddev, float()} |
+          list({time_uniform_min, latency_min_time_milliseconds()} |
+               {time_uniform_max, latency_max_time_milliseconds()} |
+               {time_gaussian_mean, latency_mean_time_milliseconds()} |
+               {time_gaussian_stddev, float() | pos_integer()} |
                {time_absolute, latency_time_milliseconds()}) | system | false} |
          {monkey_chaos,
           list({probability_request, float()} |
@@ -367,7 +428,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {aspects_init_after, list(aspect_init_after_internal())} |
@@ -382,7 +443,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {request_pid_uses, infinity | pos_integer()} |
@@ -391,7 +452,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {info_pid_uses, infinity | pos_integer()} |
@@ -400,7 +461,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {duo_mode, boolean()} |
@@ -432,12 +493,19 @@
                {count_min, number()}) | false} |
          {timeout_terminate,
           undefined | timeout_terminate_milliseconds()} |
+         {restart_delay,
+          list({time_exponential_min, restart_delay_milliseconds()} |
+               {time_exponential_max, restart_delay_milliseconds()} |
+               {time_linear_min, restart_delay_milliseconds()} |
+               {time_linear_slope, restart_delay_milliseconds()} |
+               {time_linear_max, restart_delay_milliseconds()} |
+               {time_absolute, restart_delay_milliseconds()}) | false} |
          {scope, atom()} |
          {monkey_latency,
-          list({time_uniform_min, latency_time_milliseconds()} |
-               {time_uniform_max, latency_time_milliseconds()} |
-               {time_gaussian_mean, latency_time_milliseconds()} |
-               {time_gaussian_stddev, float()} |
+          list({time_uniform_min, latency_min_time_milliseconds()} |
+               {time_uniform_max, latency_max_time_milliseconds()} |
+               {time_gaussian_mean, latency_mean_time_milliseconds()} |
+               {time_gaussian_stddev, float() | pos_integer()} |
                {time_absolute, latency_time_milliseconds()}) | system | false} |
          {monkey_chaos,
           list({probability_request, float()} |
@@ -448,7 +516,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {aspects_init_after, list(aspect_init_after_external())} |
@@ -474,9 +542,9 @@
                   {dest_refresh, dest_refresh()} |
                   {protocol, 'default' | 'local' | 'tcp' | 'udp'} |
                   {buffer_size, 'default' | pos_integer()} |
-                  {timeout_init, timeout_milliseconds()} |
-                  {timeout_async, timeout_milliseconds()} |
-                  {timeout_sync, timeout_milliseconds()} |
+                  {timeout_init, timeout_initialize_value_milliseconds()} |
+                  {timeout_async, timeout_send_async_value_milliseconds()} |
+                  {timeout_sync, timeout_send_sync_value_milliseconds()} |
                   {dest_list_deny, dest_list()} |
                   {dest_list_allow, dest_list()} |
                   {count_process, pos_integer() | float()} |
@@ -513,9 +581,9 @@
                   {code_paths_add, list(string())} |
                   {code_paths_remove, list(string())} |
                   {dest_refresh, dest_refresh()} |
-                  {timeout_init, timeout_milliseconds()} |
-                  {timeout_async, timeout_milliseconds()} |
-                  {timeout_sync, timeout_milliseconds()} |
+                  {timeout_init, timeout_initialize_milliseconds()} |
+                  {timeout_async, timeout_send_async_milliseconds()} |
+                  {timeout_sync, timeout_send_sync_milliseconds()} |
                   {dest_list_deny, dest_list()} |
                   {dest_list_allow, dest_list()} |
                   {options, service_update_plan_options_internal()}).
@@ -530,9 +598,9 @@
                   {code_paths_add, list(string())} |
                   {code_paths_remove, list(string())} |
                   {dest_refresh, dest_refresh()} |
-                  {timeout_init, timeout_milliseconds()} |
-                  {timeout_async, timeout_milliseconds()} |
-                  {timeout_sync, timeout_milliseconds()} |
+                  {timeout_init, timeout_initialize_milliseconds()} |
+                  {timeout_async, timeout_send_async_milliseconds()} |
+                  {timeout_sync, timeout_send_sync_milliseconds()} |
                   {dest_list_deny, dest_list()} |
                   {dest_list_allow, dest_list()} |
                   {options, service_update_plan_options_external()}).
@@ -553,10 +621,10 @@
          {response_timeout_immediate_max,
           response_timeout_immediate_max_milliseconds()} |
          {monkey_latency,
-          list({time_uniform_min, latency_time_milliseconds()} |
-               {time_uniform_max, latency_time_milliseconds()} |
-               {time_gaussian_mean, latency_time_milliseconds()} |
-               {time_gaussian_stddev, float()} |
+          list({time_uniform_min, latency_min_time_milliseconds()} |
+               {time_uniform_max, latency_max_time_milliseconds()} |
+               {time_gaussian_mean, latency_mean_time_milliseconds()} |
+               {time_gaussian_stddev, float() | pos_integer()} |
                {time_absolute, latency_time_milliseconds()}) | system | false} |
          {monkey_chaos,
           list({probability_request, float()} |
@@ -566,7 +634,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {aspects_init_after, list(aspect_init_after_internal())} |
@@ -580,7 +648,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {request_pid_uses, infinity | pos_integer()} |
@@ -589,7 +657,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {info_pid_uses, infinity | pos_integer()} |
@@ -598,7 +666,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {hibernate,
@@ -622,10 +690,10 @@
          {response_timeout_immediate_max,
           response_timeout_immediate_max_milliseconds()} |
          {monkey_latency,
-          list({time_uniform_min, latency_time_milliseconds()} |
-               {time_uniform_max, latency_time_milliseconds()} |
-               {time_gaussian_mean, latency_time_milliseconds()} |
-               {time_gaussian_stddev, float()} |
+          list({time_uniform_min, latency_min_time_milliseconds()} |
+               {time_uniform_max, latency_max_time_milliseconds()} |
+               {time_gaussian_mean, latency_mean_time_milliseconds()} |
+               {time_gaussian_stddev, float() | pos_integer()} |
                {time_absolute, latency_time_milliseconds()}) | system | false} |
          {monkey_chaos,
           list({probability_request, float()} |
@@ -635,7 +703,7 @@
                {fullsweep_after, non_neg_integer()} |
                {min_heap_size, non_neg_integer()} |
                {min_bin_vheap_size, non_neg_integer()} |
-               {max_heap_size, non_neg_integer() | #{}} |
+               {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap | mixed})} |
          {aspects_init_after, list(aspect_init_after_external())} |
@@ -682,7 +750,7 @@
          Module :: module(),
          Line :: pos_integer(),
          Function :: atom() | undefined,
-         Arity :: non_neg_integer() | undefined,
+         Arity :: arity() | undefined,
          MetaData :: list({atom(), any()}),
          LogMessage :: string()) ->
         ok).
@@ -699,14 +767,30 @@
               aspect_log_after/0]).
 -type loglevel() :: loglevel_on() | off.
 -type loglevel_on() :: fatal | error | warn | info | debug | trace.
+-type logging_syslog_identity() :: nonempty_string().
+-type logging_syslog_facility() :: 
+    kernel | user | mail | daemon | auth0 | syslog |
+    print | news | uucp | clock0 | auth1 | ftp | ntp |
+    auth2 | auth3 | clock1 | local0 | local1 | local2 |
+    local3 | local4 | local5 | local6 | local7 | non_neg_integer() |
+    % common aliases
+    auth | authpriv | cron | kern | lpr | security.
+-type logging_syslog_transport() :: local | udp | tcp | tls.
+-type logging_syslog_transport_options() :: list().
+-type logging_syslog_protocol() :: rfc3164 | rfc5424.
+-type logging_syslog_path() :: nonempty_string().
+-type logging_syslog_host() :: inet:ip_address() | inet:hostname().
+-type logging_syslog_port() :: undefined | inet:port_number().
 -type logging_syslog_set_proplist() ::
-    list({identity, string()} |
-         {facility, kern | user | mail | daemon | auth | syslog | lpr |
-                    news | uucp | cron | authpriv | ftp | netinfo |
-                    remoteauth | install | ras |
-                    local0 | local1 | local2 | local3 | local4 |
-                    local5 | local6 | local7 | non_neg_integer()} |
-         {level, loglevel() | undefined}).
+    list({identity, logging_syslog_identity()} |
+         {facility, logging_syslog_facility()} |
+         {level, loglevel() | undefined} |
+         {transport, logging_syslog_transport()} |
+         {transport_options, logging_syslog_transport_options()} |
+         {protocol, logging_syslog_protocol()} |
+         {path, logging_syslog_path()} |
+         {host, logging_syslog_host()} |
+         {port, logging_syslog_port()}).
 -type logging_formatters_set_proplist() ::
     list({any | nonempty_list(module()),
           list(fatal | error | warn | info | debug | trace |
@@ -729,6 +813,14 @@
                   {aspects_log_after, list(aspect_log_after())}).
 -export_type([loglevel/0,
               loglevel_on/0,
+              logging_syslog_identity/0,
+              logging_syslog_facility/0,
+              logging_syslog_transport/0,
+              logging_syslog_transport_options/0,
+              logging_syslog_protocol/0,
+              logging_syslog_path/0,
+              logging_syslog_host/0,
+              logging_syslog_port/0,
               logging_syslog_set_proplist/0,
               logging_formatters_set_proplist/0,
               logging_proplist/0]).
@@ -1368,32 +1460,6 @@ code_path(Timeout)
           (Timeout =:= infinity)) ->
     {ok, code:get_path()}.
 
-%%-------------------------------------------------------------------------
-%% @doc
-%% @deprecated Use {@link logging_level_set/2} instead
-%% @end
-%%-------------------------------------------------------------------------
-
--spec loglevel_set(Level :: loglevel(),
-                   Timeout :: api_timeout_milliseconds()) ->
-    ok.
-
-loglevel_set(Level, Timeout) ->
-    logging_level_set(Level, Timeout).
-
-%%-------------------------------------------------------------------------
-%% @doc
-%% @deprecated Use {@link logging_redirect_set/2} instead
-%% @end
-%%-------------------------------------------------------------------------
-
--spec log_redirect(Node :: undefined | node(),
-                   Timeout :: api_timeout_milliseconds()) ->
-    ok.
-
-log_redirect(Node, Timeout) ->
-    logging_redirect_set(Node, Timeout).
-
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
