@@ -7,7 +7,7 @@
 %%%
 %%% BSD LICENSE
 %%% 
-%%% Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2014-2017, Michael Truog <mjtruog at gmail dot com>
 %%% All rights reserved.
 %%% 
 %%% Redistribution and use in source and binary forms, with or without
@@ -42,8 +42,8 @@
 %%% DAMAGE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2014-2017 Michael Truog
+%%% @version 1.5.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_SUITE).
@@ -564,13 +564,13 @@ t_service_internal_sync_2(_Config) ->
     {{ok, {_, Service}},
      Context2} = cloudi:get_pid(Context1,
                                 ServiceName,
-                                immediate),
+                                limit_min),
     true = erlang:is_process_alive(Service),
     {{error, timeout},
      _Context3} = cloudi:send_sync(Context2,
                                    ServiceName,
                                    ?REQUEST_INFO7, ?REQUEST7,
-                                   immediate, undefined),
+                                   limit_min, undefined),
     receive after 100 -> ok end,
     false = erlang:is_process_alive(Service),
     ok.
@@ -770,7 +770,7 @@ t_service_internal_aspects_1(_Config) ->
     {{ok, {_, Service}},
      Context3} = cloudi:get_pid(Context2,
                                 ServiceName,
-                                immediate),
+                                limit_min),
     % count == 35 (29 + 6), InfoBefore
     Service ! increment, % count == 37 (35 + 2)
     % count == 43 (37 + 6), InfoAfter
@@ -815,7 +815,7 @@ t_service_internal_terminate_1(_Config) ->
     {{ok, {_, Service}},
      _Context2} = cloudi:get_pid(Context1,
                                  ServiceName,
-                                 immediate),
+                                 limit_min),
     erlang:exit(Service, terminate_sleep_test_1),
     % wait for termination
     receive after 4000 -> ok end,
@@ -837,7 +837,7 @@ t_service_internal_terminate_3(_Config) ->
     {{ok, {_, Service}},
      _Context2} = cloudi:get_pid(Context1,
                                  ServiceName,
-                                 immediate),
+                                 limit_min),
     erlang:exit(Service, terminate_sleep_test_2),
     % wait for termination
     receive after 4000 -> ok end,
@@ -859,9 +859,10 @@ t_service_internal_update_1(Config) ->
     {ok, [{ServiceId, _}]} = cloudi_service_api:services(infinity),
     [ServiceId] = ?config(service_ids, Config),
     ModuleState0 = fun(ModuleVersion, ModuleVersion,
-                       #state{count = Count0} = OldState) ->
+                       #state{count = CountValue0} = OldState) ->
         true = [?VSN] == ModuleVersion,
-        {ok, OldState#state{count = Count0 + 100}}
+        true = Count0 == CountValue0,
+        {ok, OldState#state{count = CountValue0 + 100}}
     end,
     % an exact service_id can be used since the module is only used once
     % (if the module was used in many services, "" or <<>> would need to be
@@ -878,8 +879,9 @@ t_service_internal_update_1(Config) ->
                                   undefined, undefined),
     100 = Count1,
     ModuleState1 = fun(ModuleVersion, ModuleVersion,
-                       #state{count = Count1}) ->
+                       #state{count = CountValue1}) ->
         true = [?VSN] == ModuleVersion,
+        true = Count1 == CountValue1,
         {error, not_updating}
     end,
     {error,
