@@ -13,7 +13,7 @@
 %%%
 %%% BSD LICENSE
 %%% 
-%%% Copyright (c) 2011-2015, Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
 %%% All rights reserved.
 %%% 
 %%% Redistribution and use in source and binary forms, with or without
@@ -48,8 +48,8 @@
 %%% DAMAGE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2015 Michael Truog
-%%% @version 1.5.1 {@date} {@time}
+%%% @copyright 2011-2016 Michael Truog
+%%% @version 1.5.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cpg_data).
@@ -195,6 +195,8 @@ get_empty_groups() ->
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get the members of a specific group.===
+%% All members are ordered from newest to oldest, based on the group
+%% membership surviving netsplits (join order, not pid creation time).
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -206,20 +208,18 @@ get_members(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
-        {ok, Pattern, #cpg_data{local = Local,
-                                remote = Remote}} ->
-            {ok, Pattern, lists:foldl(fun(#cpg_data_pid{pid = Pid}, L) ->
-                [Pid | L]
-            end, [], Remote ++ Local)}
+        {ok, Pattern, #cpg_data{history = History}} ->
+            {ok, Pattern, History}
     end.
 
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get the members of a specific group while excluding a specific pid.===
+%% All members are ordered from newest to oldest, based on the group
+%% membership surviving netsplits (join order, not pid creation time).
 %% Usually the self() pid is excluded with this function call.
 %% @end
 %%-------------------------------------------------------------------------
@@ -234,20 +234,12 @@ get_members(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
-        {ok, Pattern, #cpg_data{local = Local,
-                                remote = Remote}} ->
-            Members = lists:foldl(fun(#cpg_data_pid{pid = Pid}, L) ->
-                if
-                    Pid =/= Exclude ->
-                        [Pid | L];
-                    true ->
-                        L
-                end
-            end, [], Remote ++ Local),
+        {ok, Pattern, #cpg_data{history = History}} ->
+            Members = [Pid || Pid <- History,
+                       Pid =/= Exclude],
             if
                 Members == [] ->
                     % to keep return values consistent with pg2
@@ -260,6 +252,8 @@ get_members(GroupName, Exclude, Groups)
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get only the local members of a specific group.===
+%% All members are ordered from newest to oldest, based on the 
+%% join order, not pid creation time.
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -275,14 +269,14 @@ get_local_members(GroupName, Groups) ->
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{local = Local}} ->
-            {ok, Pattern, lists:foldl(fun(#cpg_data_pid{pid = Pid}, L) ->
-                [Pid | L]
-            end, [], Local)}
+            {ok, Pattern, [Pid || #cpg_data_pid{pid = Pid} <- Local]}
     end.
 
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get only the local members of a specific group while excluding a specific pid.===
+%% All members are ordered from newest to oldest, based on the 
+%% join order, not pid creation time.
 %% Usually the self() pid is excluded with this function call.
 %% @end
 %%-------------------------------------------------------------------------
@@ -301,14 +295,8 @@ get_local_members(GroupName, Exclude, Groups)
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{local = Local}} ->
-            Members = lists:foldl(fun(#cpg_data_pid{pid = Pid}, L) ->
-                if
-                    Pid =/= Exclude ->
-                        [Pid | L];
-                    true ->
-                        L
-                end
-            end, [], Local),
+            Members = [Pid || #cpg_data_pid{pid = Pid} <- Local,
+                       Pid =/= Exclude],
             if
                 Members == [] ->
                     % to keep return values consistent with pg2
@@ -321,6 +309,8 @@ get_local_members(GroupName, Exclude, Groups)
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get only the remote members of a specific group.===
+%% All members are ordered from newest to oldest, based on the group
+%% membership surviving netsplits (join order, not pid creation time).
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -336,14 +326,14 @@ get_remote_members(GroupName, Groups) ->
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{remote = Remote}} ->
-            {ok, Pattern, lists:foldl(fun(#cpg_data_pid{pid = Pid}, L) ->
-                [Pid | L]
-            end, [], Remote)}
+            {ok, Pattern, [Pid || #cpg_data_pid{pid = Pid} <- Remote]}
     end.
 
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get only the remote members of a specific group while excluding a specific pid.===
+%% All members are ordered from newest to oldest, based on the group
+%% membership surviving netsplits (join order, not pid creation time).
 %% Usually the self() pid is excluded with this function call.
 %% @end
 %%-------------------------------------------------------------------------
@@ -362,14 +352,8 @@ get_remote_members(GroupName, Exclude, Groups)
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{remote = Remote}} ->
-            Members = lists:foldl(fun(#cpg_data_pid{pid = Pid}, L) ->
-                if
-                    Pid =/= Exclude ->
-                        [Pid | L];
-                    true ->
-                        L
-                end
-            end, [], Remote),
+            Members = [Pid || #cpg_data_pid{pid = Pid} <- Remote,
+                       Pid =/= Exclude],
             if
                 Members == [] ->
                     % to keep return values consistent with pg2
@@ -406,8 +390,7 @@ get_closest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{local_count = 0,
                                 remote_count = RemoteCount,
@@ -436,8 +419,7 @@ get_closest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{local_count = 0,
                                 remote_count = RemoteCount,
@@ -467,8 +449,7 @@ get_furthest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{remote_count = 0,
-                          local_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{remote_count = 0,
                                 local_count = LocalCount,
@@ -497,8 +478,7 @@ get_furthest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{remote_count = 0,
-                          local_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{remote_count = 0,
                                 local_count = LocalCount,
@@ -527,8 +507,7 @@ get_random_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{local_count = LocalCount,
                                 local = Local,
@@ -555,8 +534,7 @@ get_random_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{local_count = LocalCount,
                                 local = Local,
@@ -675,11 +653,10 @@ get_oldest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            {ok, Pattern, history_oldest(History)}
+            history_oldest(History, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -700,16 +677,10 @@ get_oldest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_oldest(History, Exclude) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_oldest(History, Exclude, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -727,16 +698,10 @@ get_local_oldest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{local_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_local_oldest(History) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_local_oldest(History, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -757,16 +722,10 @@ get_local_oldest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{local_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_local_oldest(History, Exclude) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_local_oldest(History, Exclude, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -784,16 +743,10 @@ get_remote_oldest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{remote_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_remote_oldest(History) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_remote_oldest(History, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -814,16 +767,10 @@ get_remote_oldest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{remote_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_remote_oldest(History, Exclude) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_remote_oldest(History, Exclude, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -841,11 +788,10 @@ get_newest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            {ok, Pattern, history_newest(History)}
+            history_newest(History, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -866,16 +812,10 @@ get_newest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{history = []}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_newest(History, Exclude) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_newest(History, Exclude, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -893,16 +833,10 @@ get_local_newest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{local_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_local_newest(History) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_local_newest(History, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -923,16 +857,10 @@ get_local_newest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{local_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_local_newest(History, Exclude) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_local_newest(History, Exclude, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -950,16 +878,10 @@ get_remote_newest_pid(GroupName, Groups) ->
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{remote_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_remote_newest(History) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_remote_newest(History, GroupName, Pattern)
     end.
 
 %%-------------------------------------------------------------------------
@@ -980,16 +902,10 @@ get_remote_newest_pid(GroupName, Exclude, Groups)
     case group_find(GroupName, Groups) of
         error ->
             {error, {'no_such_group', GroupName}};
-        {ok, _, #cpg_data{local_count = 0,
-                          remote_count = 0}} ->
+        {ok, _, #cpg_data{remote_count = 0}} ->
             {error, {'no_process', GroupName}};
         {ok, Pattern, #cpg_data{history = History}} ->
-            case history_remote_newest(History, Exclude) of
-                undefined ->
-                    {error, {'no_process', GroupName}};
-                Pid ->
-                    {ok, Pattern, Pid}
-            end
+            history_remote_newest(History, Exclude, GroupName, Pattern)
     end.
 
 %%%------------------------------------------------------------------------
@@ -1066,121 +982,129 @@ pick(N1, L1, N2, L2, Exclude, GroupName, Pattern) ->
             Success
     end.
 
+history_oldest([_ | _] = History, Pattern) ->
+    [Pid | _] = lists:reverse(History),
+    {ok, Pattern, Pid}.
+
+history_oldest_pid([], _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_oldest_pid([Exclude | History], Exclude, GroupName, Pattern) ->
+    history_oldest_pid(History, Exclude, GroupName, Pattern);
+history_oldest_pid([Pid | _], _, _, Pattern) ->
+    {ok, Pattern, Pid}.
+history_oldest([_ | _] = History, Exclude, GroupName, Pattern) ->
+    history_oldest_pid(lists:reverse(History), Exclude, GroupName, Pattern).
+
+history_local_oldest_pid([], _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_local_oldest_pid([Pid | _], Node, _, Pattern)
+    when node(Pid) =:= Node ->
+    {ok, Pattern, Pid};
+history_local_oldest_pid([_ | History], Node, GroupName, Pattern) ->
+    history_local_oldest_pid(History, Node, GroupName, Pattern).
+history_local_oldest([_ | _] = History, GroupName, Pattern) ->
+    history_local_oldest_pid(lists:reverse(History), node(),
+                             GroupName, Pattern).
+
+history_local_oldest_pid([], _, _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_local_oldest_pid([Exclude | History], Exclude, Node,
+                         GroupName, Pattern) ->
+    history_local_oldest_pid(History, Exclude, Node, GroupName, Pattern);
+history_local_oldest_pid([Pid | _], _, Node, _, Pattern)
+    when node(Pid) =:= Node ->
+    {ok, Pattern, Pid};
+history_local_oldest_pid([_ | History], Exclude, Node, GroupName, Pattern) ->
+    history_local_oldest_pid(History, Exclude, Node, GroupName, Pattern).
+history_local_oldest([_ | _] = History, Exclude, GroupName, Pattern) ->
+    history_local_oldest_pid(lists:reverse(History), Exclude, node(),
+                             GroupName, Pattern).
+
+history_remote_oldest_pid([], _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_remote_oldest_pid([Pid | _], Node, _, Pattern)
+    when node(Pid) =/= Node ->
+    {ok, Pattern, Pid};
+history_remote_oldest_pid([_ | History], Node, GroupName, Pattern) ->
+    history_remote_oldest_pid(History, Node, GroupName, Pattern).
+history_remote_oldest([_ | _] = History, GroupName, Pattern) ->
+    history_remote_oldest_pid(lists:reverse(History), node(),
+                              GroupName, Pattern).
+
+history_remote_oldest_pid([], _, _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_remote_oldest_pid([Exclude | History], Exclude, Node,
+                          GroupName, Pattern) ->
+    history_remote_oldest_pid(History, Exclude, Node, GroupName, Pattern);
+history_remote_oldest_pid([Pid | _], _, Node, _, Pattern)
+    when node(Pid) =/= Node ->
+    {ok, Pattern, Pid};
+history_remote_oldest_pid([_ | History], Exclude, Node, GroupName, Pattern) ->
+    history_remote_oldest_pid(History, Exclude, Node, GroupName, Pattern).
+history_remote_oldest([_ | _] = History, Exclude, GroupName, Pattern) ->
+    history_remote_oldest_pid(lists:reverse(History), Exclude, node(),
+                              GroupName, Pattern).
+
+history_newest([Pid | _], Pattern) ->
+    {ok, Pattern, Pid}.
+
+history_newest_pid([], _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_newest_pid([Exclude | History], Exclude, GroupName, Pattern) ->
+    history_newest_pid(History, Exclude, GroupName, Pattern);
+history_newest_pid([Pid | _], _, _, Pattern) ->
+    {ok, Pattern, Pid}.
+history_newest([_ | _] = History, Exclude, GroupName, Pattern) ->
+    history_newest_pid(History, Exclude, GroupName, Pattern).
+
+history_local_newest_pid([], _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_local_newest_pid([Pid | _], Node, _, Pattern)
+    when node(Pid) =:= Node ->
+    {ok, Pattern, Pid};
+history_local_newest_pid([_ | History], Node, GroupName, Pattern) ->
+    history_local_newest_pid(History, Node, GroupName, Pattern).
+history_local_newest([_ | _] = History, GroupName, Pattern) ->
+    history_local_newest_pid(History, node(), GroupName, Pattern).
+
+history_local_newest_pid([], _, _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_local_newest_pid([Exclude | History], Exclude, Node,
+                         GroupName, Pattern) ->
+    history_local_newest_pid(History, Exclude, Node, GroupName, Pattern);
+history_local_newest_pid([Pid | _], _, Node, _, Pattern)
+    when node(Pid) =:= Node ->
+    {ok, Pattern, Pid};
+history_local_newest_pid([_ | History], Exclude, Node, GroupName, Pattern) ->
+    history_local_newest_pid(History, Exclude, Node, GroupName, Pattern).
+history_local_newest([_ | _] = History, Exclude, GroupName, Pattern) ->
+    history_local_newest_pid(History, Exclude, node(), GroupName, Pattern).
+
+history_remote_newest_pid([], _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_remote_newest_pid([Pid | _], Node, _, Pattern)
+    when node(Pid) =/= Node ->
+    {ok, Pattern, Pid};
+history_remote_newest_pid([_ | History], Node, GroupName, Pattern) ->
+    history_remote_newest_pid(History, Node, GroupName, Pattern).
+history_remote_newest([_ | _] = History, GroupName, Pattern) ->
+    history_remote_newest_pid(History, node(), GroupName, Pattern).
+
+history_remote_newest_pid([], _, _, GroupName, _) ->
+    {error, {'no_process', GroupName}};
+history_remote_newest_pid([Exclude | History], Exclude, Node,
+                          GroupName, Pattern) ->
+    history_remote_newest_pid(History, Exclude, Node, GroupName, Pattern);
+history_remote_newest_pid([Pid | _], _, Node, _, Pattern)
+    when node(Pid) =/= Node ->
+    {ok, Pattern, Pid};
+history_remote_newest_pid([_ | History], Exclude, Node, GroupName, Pattern) ->
+    history_remote_newest_pid(History, Exclude, Node, GroupName, Pattern).
+history_remote_newest([_ | _] = History, Exclude, GroupName, Pattern) ->
+    history_remote_newest_pid(History, Exclude, node(), GroupName, Pattern).
+
 -compile({inline, [{random,1}]}).
 
 random(N) ->
     quickrand:uniform(N).
 
-history_oldest([_ | _] = History) ->
-    [Pid | _] = lists:reverse(History),
-    Pid.
-
-history_oldest_pid([], _) ->
-    undefined;
-history_oldest_pid([Exclude | History], Exclude) ->
-    history_oldest_pid(History, Exclude);
-history_oldest_pid([Pid | _], _) ->
-    Pid.
-history_oldest([_ | _] = History, Exclude) ->
-    history_oldest_pid(lists:reverse(History), Exclude).
-
-history_local_oldest_pid([]) ->
-    undefined;
-history_local_oldest_pid([Pid | _])
-    when node(Pid) == node() ->
-    Pid;
-history_local_oldest_pid([_ | History]) ->
-    history_local_oldest_pid(History).
-history_local_oldest([_ | _] = History) ->
-    history_local_oldest_pid(lists:reverse(History)).
-
-history_local_oldest_pid([], _) ->
-    undefined;
-history_local_oldest_pid([Exclude | History], Exclude) ->
-    history_local_oldest_pid(History, Exclude);
-history_local_oldest_pid([Pid | _], _)
-    when node(Pid) == node() ->
-    Pid;
-history_local_oldest_pid([_ | History], Exclude) ->
-    history_local_oldest_pid(History, Exclude).
-history_local_oldest([_ | _] = History, Exclude) ->
-    history_local_oldest_pid(lists:reverse(History), Exclude).
-
-history_remote_oldest_pid([]) ->
-    undefined;
-history_remote_oldest_pid([Pid | _])
-    when node(Pid) /= node() ->
-    Pid;
-history_remote_oldest_pid([_ | History]) ->
-    history_remote_oldest_pid(History).
-history_remote_oldest([_ | _] = History) ->
-    history_remote_oldest_pid(lists:reverse(History)).
-
-history_remote_oldest_pid([], _) ->
-    undefined;
-history_remote_oldest_pid([Exclude | History], Exclude) ->
-    history_remote_oldest_pid(History, Exclude);
-history_remote_oldest_pid([Pid | _], _)
-    when node(Pid) /= node() ->
-    Pid;
-history_remote_oldest_pid([_ | History], Exclude) ->
-    history_remote_oldest_pid(History, Exclude).
-history_remote_oldest([_ | _] = History, Exclude) ->
-    history_remote_oldest_pid(lists:reverse(History), Exclude).
-
-history_newest([Pid | _]) ->
-    Pid.
-
-history_newest_pid([], _) ->
-    undefined;
-history_newest_pid([Exclude | History], Exclude) ->
-    history_newest_pid(History, Exclude);
-history_newest_pid([Pid | _], _) ->
-    Pid.
-history_newest([_ | _] = History, Exclude) ->
-    history_newest_pid(History, Exclude).
-
-history_local_newest_pid([]) ->
-    undefined;
-history_local_newest_pid([Pid | _])
-    when node(Pid) == node() ->
-    Pid;
-history_local_newest_pid([_ | History]) ->
-    history_local_newest_pid(History).
-history_local_newest([_ | _] = History) ->
-    history_local_newest_pid(History).
-
-history_local_newest_pid([], _) ->
-    undefined;
-history_local_newest_pid([Exclude | History], Exclude) ->
-    history_local_newest_pid(History, Exclude);
-history_local_newest_pid([Pid | _], _)
-    when node(Pid) == node() ->
-    Pid;
-history_local_newest_pid([_ | History], Exclude) ->
-    history_local_newest_pid(History, Exclude).
-history_local_newest([_ | _] = History, Exclude) ->
-    history_local_newest_pid(History, Exclude).
-
-history_remote_newest_pid([]) ->
-    undefined;
-history_remote_newest_pid([Pid | _])
-    when node(Pid) /= node() ->
-    Pid;
-history_remote_newest_pid([_ | History]) ->
-    history_remote_newest_pid(History).
-history_remote_newest([_ | _] = History) ->
-    history_remote_newest_pid(History).
-
-history_remote_newest_pid([], _) ->
-    undefined;
-history_remote_newest_pid([Exclude | History], Exclude) ->
-    history_remote_newest_pid(History, Exclude);
-history_remote_newest_pid([Pid | _], _)
-    when node(Pid) /= node() ->
-    Pid;
-history_remote_newest_pid([_ | History], Exclude) ->
-    history_remote_newest_pid(History, Exclude).
-history_remote_newest([_ | _] = History, Exclude) ->
-    history_remote_newest_pid(History, Exclude).
-
