@@ -56,7 +56,7 @@
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2010-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @version 1.5.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(trie).
@@ -839,19 +839,32 @@ pattern_parse(Pattern, L) ->
 
 -spec pattern_parse(Pattern :: string(),
                     L :: string(),
-                    Option :: default | with_suffix) ->
-    list(string()) | {list(string()), string()} | 'error'.
+                    Option :: default | with_suffix | expanded) ->
+    list(string()) |                       % default
+    {list(string()), string()} |           % with_suffix
+    list(string() | {exact, string()}) |   % expanded
+    'error'.
 
 pattern_parse(Pattern, L, Option)
     when (Option =:= default) orelse
-         (Option =:= with_suffix) ->
+         (Option =:= with_suffix) orelse
+         (Option =:= expanded) ->
     pattern_parse(Pattern, L, [], [], Option).
 
 pattern_parse_result(default, Parameters, _) ->
-    Parameters;
+    lists:reverse(Parameters);
 
 pattern_parse_result(with_suffix, Parameters, Suffix) ->
-    {Parameters, lists:reverse(Suffix)}.
+    {lists:reverse(Parameters), lists:reverse(Suffix)};
+
+pattern_parse_result(expanded, Parameters, Suffix) ->
+    NewParameters = if
+        Suffix /= [] ->
+            [{exact, lists:reverse(Suffix)} | Parameters];
+        true ->
+            Parameters
+    end,
+    lists:reverse(NewParameters).
 
 pattern_parse_element(_, [], _) ->
     error;
@@ -882,7 +895,7 @@ pattern_parse_pattern(Pattern, C, L, Segment, Parsed, Option) ->
     end.
 
 pattern_parse([], [], Parsed, Suffix, Option) ->
-    pattern_parse_result(Option, lists:reverse(Parsed), Suffix);
+    pattern_parse_result(Option, Parsed, Suffix);
 
 pattern_parse([], [_ | _], _, _, _) ->
     error;
@@ -890,14 +903,26 @@ pattern_parse([], [_ | _], _, _, _) ->
 pattern_parse([_ | _], [$* | _], _, _, _) ->
     erlang:exit(badarg);
 
-pattern_parse([$*], [_ | _] = L, Parsed, _, Option) ->
-    pattern_parse_result(Option, lists:reverse([L | Parsed]), []);
+pattern_parse([$*], [_ | _] = L, Parsed, Suffix, Option) ->
+    NewParsed = if
+        Option =:= expanded, Suffix /= [] ->
+            [{exact, lists:reverse(Suffix)} | Parsed];
+        true ->
+            Parsed
+    end,
+    pattern_parse_result(Option, [L | NewParsed], []);
 
 pattern_parse([$*, $* | _], [_ | _], _, _, _) ->
     erlang:exit(badarg);
 
-pattern_parse([$*, C | Pattern], [H | T], Parsed, _, Option) ->
-    pattern_parse_pattern(Pattern, C, T, [H], Parsed, Option);
+pattern_parse([$*, C | Pattern], [H | T], Parsed, Suffix, Option) ->
+    NewParsed = if
+        Option =:= expanded, Suffix /= [] ->
+            [{exact, lists:reverse(Suffix)} | Parsed];
+        true ->
+            Parsed
+    end,
+    pattern_parse_pattern(Pattern, C, T, [H], NewParsed, Option);
 
 pattern_parse([C | Pattern], [C | L], Parsed, Suffix, Option) ->
     pattern_parse(Pattern, L, Parsed, [C | Suffix], Option);
@@ -1236,6 +1261,13 @@ test() ->
     {ok,"*/",empty} = trie:find_match("///", trie:new(["*/"])),
     [] = trie:pattern_parse("aaabb", "aaabb"),
     {[], "aaabb"} = trie:pattern_parse("aaabb", "aaabb", with_suffix),
+    [{exact, "a"},
+     "ddi",
+     {exact, "t"},
+     "io",
+     {exact, "n"}] = trie:pattern_parse("a*t*n", "addition", expanded),
+    ["w",{exact,"atch"}] = trie:pattern_parse("*atch", "watch", expanded),
+    [{exact,"is"},"t"] = trie:pattern_parse("is*", "ist", expanded),
     false = trie:is_pattern("abcdef"),
     true = trie:is_pattern("abc*d*ef"),
     {'EXIT',badarg} = (catch trie:is_pattern("abc**ef")),
